from random import *

# This class is a score map used by some of the ai players
class ScoreMap:

	def __init__(self):
		""" New and `improved'"""
		self.scoremap = [
			[3,4,5,7,5,4,3],
			[4,6,8,10,8,6,4],
			[5,8,12,13,12,8,5],
			[5,8,12,13,12,8,5],
			[4,6,8,10,8,6,4],
			[3,4,5,7,5,4,3]
		]
	
	# Get the score for a single stone
	def getStoneScore(self, x, y):
		# Reverse the x and y because the map is in a human readable form:
		# 6 x 7 horizontal array's
		return self.scoremap[y][x] 

		# Get the score for a single stone
	def setStoneScore(self, x, y, value):
		# Reverse the x and y because the map is in a human readable form:
		# 6 x 7 horizontal array's
		self.scoremap[y][x] = value

	# These are not implemented, there was no need for them anymore...yet?
	def loadScoreMap(file_name):
		return 0

	def saveScoreMap(file_name):
		return 0
	
	def updateScoreMap(board, player, win):
		return 0

	def randomizeMap(self):
		x_center = int (random()*5)
		y_center = int (random()*4)
		for x in [-1,0,1]:
			for y in [-1,0,1]:
				self.scoremap[y+y_center][x+x_center] = + int(random()*2)




###############################################################################################
# $Id: label.py,v 1.3 2002/02/26 15:35:56 slmm Exp $
###############################################################################################

import pygame
from pygame.locals import *
from widget import Widget
import widget

class Label(Widget):
    """
    This class defines a simple label with a text rendered in a certain font. 
    """

    def __init__ (self, font, text, position = (0,0), callbacks = None,
                  color = (255,255,255), background = (0,0,0), background_image = None):
        """Initializes the widget. Renders the font using the passed data."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        # create the surface
        self.surface = font.render ( text, 1, color )

        # store the needed data so that we can set the text later
        self.font = font
        self.color = color
        self.background = background
	self.background_img = background_image

        # store our text too
        self.text = text


    def setText (self, text):
        """Sets a new text for the label. Renders the new label using the font and colors passed in
        the constrctor."""



        # create the surface
	try:
		textsurface = self.font.render( text, 1, self.color )
		self.surface = textsurface
		if self.background_img:
			self.surface.blit(self.background_img,(0,0))

		# store the new text
		self.text = text

		# we're dirty now
		self.dirty = 1
	except:
		return

    def getText (self):
        """Returns the current text of the label."""
        return self.text



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



# Echo server program
import socket
import select

HOST = ''                 # Symbolic name meaning the local host
PORT = 50007              # Arbitrary non-privileged port
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.bind((HOST, PORT))
s.listen(1)

print select.select([s.fileno()],[],[],30)
conn, addr = s.accept()
print 'Connected by', addr
while 1:
    data = conn.recv(1024)
    if not data: break
    conn.send(data)
conn.close()

	    



from gui.dialog import *
from gui.image import *
from gui.checkbox import *
from gui.editfield import *
import socket

# The chat class, id implements all the chat crap i need
class Chat(Dialog):
	def __init__(self, surface, images, locations, host=None):
		self.images = images
                self.locations = locations

		if host is None:
			s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			s.bind(('',50001))
			s.listen(1)
			self.connection, self.address = s.accept()
		else:
			self.connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.connection.connect((host, 50001))

		self.connection.setblocking(0)

		surface.blit(self.images['startscreen']['background'],(0,0))

		Dialog.__init__(self, surface)
		# we want timer events
		self.enableTimer ( 3000 )
		#Dialog.run(self)


	def createWidgets(self):
		# Set the background
		#self.surface.blit(self.images['chat']['background'],(0,0))

		self.wm.register(Button(self.images['chat']['send'], self.images['chat']['send'],
			(int(self.locations['chat']['send_x']),
                        int(self.locations['chat']['send_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.send }
		))

		self.r_message = Label(pygame.font.Font('fonts/lucida.ttf', 14),"appeltaart",
			(int(self.locations['chat']['message_x']),
                        int(self.locations['chat']['message_y']))
		)

		self.wm.register(self.r_message)

		self.s_message = EditField( pygame.font.Font("fonts/lucida.ttf", 14), text = "slm.ath.cx", width = 200, position = (10,20),
			cursor = self.images['gui']['cursor'], frameicons = self.images['frame'])
		self.wm.register(self.s_message)
 
	def receive(self):
		try:
			text = self.connection.recv(1024)
		except:
			text = self.r_message.getText()

		self.r_message.setText(text)
		#print text

	def send(self, trigger, event):
		text = self.s_message.getText()
		self.connection.send(str(text))
		self.s_message.setText(" ")

	def timer (self):
		self.receive()



import rules
from player import *
from random import *
import copy
from board import *

class Node:
	def __init__(self, board, parent, move, player):
		self.board = board
		self.parent = parent
		self.value = 1
		self.childs = []
		self.move = move
		self.player = player

		self.wins_score = 0
		self.lose_score = 0

	def __repr__(self):
		if self.childs > 1:
			number=0
			for child in self.childs:
				number=number+1
			return "Has %s child(s)" % number
		return "STATE"	

class Agressive(Player):
	type = 'AI'

	def __init__(self, screen, images, settings):
                self.screen = screen
                self.images = images
                self.settings = settings
		self.search_depth = int(settings.getGlobal()['game']['ailevel'])

	def evaluate(self, node, player, opponent, depth):
		if rules.isWinner(node.board, opponent):
	                node.value = -100 / (depth + 1) 
	        elif rules.isWinner(node.board, player):
	                node.value = 100 / (depth + 1)
		
		elif len(node.childs) > 0:
			list = []
			for child in node.childs:
				list.append(child.value)
			if node.player == player:
				node.value = min(list)
			else:
				node.value = max(list)
                else:
			node.value = self.score(node, player, opponent)  / (depth + 1)

		# The "argessive" way of evaluation
		if len(node.childs) > 0:
			for child in node.childs:
				node.wins_score = node.wins_score + child.wins_score
				node.lose_score += child.lose_score
				# Check if it is a win
				if child.value > 0:
					node.wins_score = node.wins_score + 1
				elif child.value < 0:
					node.lose_score += 1

	def score(self, node, player, opponent):
		# Not relevant for this function
		return 0
	
	
	def makeBoard(self, move, board, player):
		temp_board = copy.deepcopy(board)
		temp_board.move(move, player)
	        return temp_board

	def listMoves(self, board, player):
		options = []
		for move in range(7):
                        if rules.isMoveLegal(board, move):
                                options.append(self.makeBoard(move, board, player))
		return options

	def statespace(self, node, depth, current_player, player, opponent):
		if self.listMoves(node.board, 0) < 1:
			self.evaluate(node, player, opponent, depth)
			return node
		elif depth < self.search_depth:
			if current_player==2:
				next_player = 1
			else:
				next_player = 2
			for move in self.listMoves(node.board, current_player):
				node.childs.append(self.statespace(Node( move, node, 0, player), depth+1, next_player, player, opponent))
		self.evaluate(node, player, opponent, depth)
		return node
		

	def doMove(self, current_board, player):
		board = copy.deepcopy(current_board)

		if player == 1: opponent = 2
		else: opponent = 1
		
		node = Node(board, "", 0, player)
			
		node =  self.statespace( node, 0, player, player, opponent);

		bestscore = -100
		wins_score = -1 
		move = 0
		print "New Round"

		for child in node.childs:
			print "Child value=", child.value, "score=", child.wins_score
			if child.value > bestscore:
				if rules.isMoveLegal(board, child.board.last_move):
					bestscore = child.value
					move = child.board.last_move
			elif child.value == bestscore:
				if child.wins_score > wins_score:
					wins_score = child.wins_score
					move = child.board.last_move

		
		
		return move



###############################################################################################
# $Id: info_scenario.py,v 1.1.1.1 2002/02/19 10:16:03 slmm Exp $
###############################################################################################

import pygame
import pygame.image
from pygame.locals import *
import scenario
import properties
import string
import widget
from dialog import *
from button import Button
from label import Label
#from info_map import InfoMap
from widget_manager import WidgetManager

class InfoScenario (Dialog):
    """
    This class is used as a dialog for showing information about the scenario. It shows the
    background information about the selected scenario. 
    """

    def __init__ (self, info):
        """Creates the dialog. The passed 'info' is a ScenarioInfo about which the data should
        be displayed."""
        
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # store the info
        self.info = info
 
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        #self.description =
        self.createDescription ( self.smallfont, color=(255, 255, 255) )

        # set our background to a tiled image
        self.setBackground ( properties.window_background )


    def createDescription (self, font, color):
        """Creates the description labels. The description may be one or more lines of free text,
        and it needs to be created into a series of labels that can be blitted out. This method
        tries to separate the text into as large lines as possible and create a list of labels."""

        # max width and starting x and y
        width = 770
        x = 200
        y = 200

        # number of lines so far
        count = 0

        # loop over all paragraphs in the description
        for para in self.info.getDescription ():
            # format the para to get rid of unwanted whitespace
            para = string.join ( para.split (), ' ' ) 
            
            # temporaries
            testline = ''
            
            # loop over all words in the paragraph
            for word in para.split ( ' ' ):
                # store the current full line
                okline = testline

                # merge a text we use to test the width with. Don't add a ' ' if we only have one word
                # so far
                if testline == '':
                    testline = word
                else:
                    testline =  testline + ' ' + word

                # get the size of the label as it would be when rendered
                sizex, sizey = font.size ( testline )
                
                # too wide?
                if sizex > width:
                    # yep, os use the last 'good' text that fits and render a label
                    label = Label (font, okline, (x, y + count * 20), color=color )
                    
                    # register it
                    self.wm.register ( label )

                    # start with a new full line that is the part that was 'too much'
                    testline = word

                    # one more label rendered
                    count += 1
        
            # still something in 'all' that has not made it into a full line? we add a last (short) line
            # with the extra text
            if testline != '':
                label = Label ( font, testline, (x, y + count * 20), color=color )
                
                # register it
                self.wm.register ( label )

                # one more label rendered, add some empty space too
                count += 2
                

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-forward-moff.png",
                                    properties.path_dialogs + "butt-forward-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.nextScreen } ) )

        # get the date and create a string
        (year, month, day, hour, minute) = self.info.getDate ()
        date = `year` + "." + `month` + "." + `day` + " " + `hour` + ":" + `minute`
        
        # labels
        self.wm.register ( Label (self.titlefont, "Scenario information", (10, 10), color=(255, 0, 0)))
        self.wm.register ( Label (self.smallfont, "Name: ", (50, 80), color=(255, 255, 0) ))
        self.wm.register ( Label (self.smallfont, "Date: ", (50, 110), color=(255, 255, 0) ))
        self.wm.register ( Label (self.smallfont, date, (200,110), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, self.info.getName (), (200, 80),
                                  color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Location: ", (50, 140),
                                  color=(255, 255, 0) ))
        self.wm.register ( Label (self.smallfont, self.info.getLocation (), (200,140),
                                  color=(255,255,255)) )
        self.wm.register ( Label (self.smallfont, "Turns: ", (50, 170),  (0,0,0),
                                  color=(255, 255, 0) ))
        self.wm.register ( Label (self.smallfont, str(self.info.getMaxTurns ()), (200, 170),
                                  color=(255, 255, 255) ))
        self.wm.register ( Label (self.smallfont, "Description:", (50, 200), color=(255, 255, 0)))



    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Simply closes the dialog."""
        # we're cancelling the dialog
        self.state = ACCEPTED

        return widget.DONE

      
    def nextScreen (self, triggered, event):
        """Callback triggered when the user clicks the 'Next' button. Shows the map info
        dialog. After the dialog has been shown this dialog is repainted."""

        print "nextScreen"
        # run the dialog
#        InfoMap ().run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: canvas.py,v 1.1.1.1 2002/02/19 10:15:58 slmm Exp $
###############################################################################################

from widget import Widget
import widget
import pygame
import pygame.image
from pygame.locals import *

class Canvas (Widget):
    """
    This class defines a...
    """

    def __init__ (self, position = (0,0), size = (1,1), color = (0,0,0,0)):
        """Initializes the widget. Loads the icons from the passed filename."""

        # firts call superclass constructor
        Widget.__init__ (self, position, None)

        # create the surface
        self.surface = pygame.Surface ( (size[0], size[1]), HWSURFACE ).convert ()
 
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: info_map.py,v 1.1.1.1 2002/02/19 10:16:05 slmm Exp $
###############################################################################################

import pygame
import pygame.image
from pygame.locals import *
import scenario
import properties
import widget
from button import Button
from info_units import InfoUnits
from widget_manager import WidgetManager

class InfoMap:
    """
    This class is used as a dialog for.
    """

    def __init__ (self):
        "Creates the dialog."

        # create a new surface matching the display
        self.surface = sdl.surface_new ( (properties.window_size_x, properties.window_size_y), 1)
        self.surface = self.surface.convert ()
        
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )

        # create the static labels
        self.title = self.titlefont.render ( "Scenario map ", 1, (255, 0, 0) )

        # blit 'em
        self.surface.blit ( self.title, (10, 10 ))

        # get the map
        map = scenario.map
        
        # get the hexes of the map
        hexes = scenario.map.getHexes ()

        # create a new surface for it. It must contain the entire map (2x2 pixels per hex) and be a
        # little wider as every even row is adjusted half a hex to the right
        self.minimap = sdl.surface_new ( (map.getXsize () * 4 + 1, map.getYsize () * 3 + 1), 1 )
        
        # loop over the hexes in the map
        for y in range ( map.getYsize () ):
            for x in range ( map.getXsize () ):
                # get the hex for that position
                hex = hexes [x][y]

                # get the miniicon for the hex
                icon = hex.getMiniIcon ()
                
                # is this an odd or even row?
                if y % 2:
                    # odd
                    self.minimap.blit ( icon, (x * 4 + 2, y * 3 ))
                else:
                    # even
                    self.minimap.blit ( icon, (x * 4, y * 3 ))


        # calculate the x position for the map
        xpos = properties.window_size_x / 2 - ( self.minimap.get_size ()[0] / 2 )
        
        # blit out the created surface
        self.surface.blit ( self.minimap, (xpos, self.title.get_size()[1] + 40 ) )

        # create the widget manager
        self.wm = WidgetManager (self.surface)

        # create all widgets
        self.createWidgets ()
      
        # blit out or full surface to the main surface
        scenario.sdl.blit ( self.surface, (0, 0 ))

        # update the whole screen
        scenario.sdl.update ()
        #pygame.display.update ()



    def createWidgets (self):
        "Creates all widgets for the dialog."
        # create the buttons for the next and previous buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-back-moff.png",
                                    properties.path_dialogs + "butt-back-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.prevScreen } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-forward-moff.png",
                                    properties.path_dialogs + "butt-forward-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.nextScreen } ) )


    def run (self):
        """Executes the dialog and runs its internal loop until a key is pressed. When that happens
        shows the dialog InfoUnits. When that dialog is terminated this method also returns."""

        # loop forever
        while 1:
            # get next event
            event = pygame.event.wait ()

            # see wether the widget manager wants to handle it
            if event != -1:
                # handle event and get the return code that tells us wehter it was handled or not
                returncode = self.wm.handle ( event )

                # is the event loop done?
                if returncode == widget.DONE:
                    return


    def nextScreen (self, event):
        """Callback triggered when the user clicks the 'Next' button. Shows the 'Order of battle'
        dialog. After the dialog has been shown this dialog is repainted."""

        # show the dialog
        InfoUnits ().run ()

        # blit out or full surface to the main surface
        scenario.sdl.blit ( self.surface, (0, 0 ))

        # update the whole screen
        scenario.sdl.update ()
        #pygame.display.update ()

        # we've handled the event
        return widget.HANDLED
    

    def prevScreen (self, event):
        """Callback triggered when the user clicks the 'Previous' button. This method simply cancels
        the event loop for this dialog's widget manager. Basically this dialog quits."""
        # we're done
        return widget.DONE
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: widget_manager.py,v 1.1.1.1 2002/02/19 10:15:59 slmm Exp $
###############################################################################################

import pygame
import pygame.display
from pygame.locals import *
from widget import Widget
import widget

class WidgetManager:
    """
    This class defines a simple framework for managing widgets and their events. All created widgets
    should be added to a framework before they are of any use or can receive events. All SDL events
    should be sent to this framework for possible dispatching to widgets. If an event does not
    consern any widget the other parts of the application are free to do whatever they wish with the
    event.

    Every event should be sent to handle() as the raw SDL event as received from pygame.event.poll ()
    or pygame.event.wait().
    """

    def __init__ (self, surface, dialog=None):
        """Initializes the instance"""

        # clear the list of registered widgets
        self.widgets = []

        # the previous mouse position, for enter/leave events
        self.last_pos = (-1, -1)

        # the surface widgets are painted on
        self.surface = surface

        # store the dialog too
        self.dialog = dialog
        

    def register (self, widget):
        """Register a widget within the widget manager. The widget will after it has been registered
        be able to receive various events."""
        # is the widget already in the list?
        if not widget in self.widgets:
            # nope, add it
            self.widgets.append ( widget )
            

    def deRegister (self, widget):
        """Removes the passed widget from the management."""
        # do we have the widget in the list?
        if widget in self.widgets:
            # yep, remove it
            self.widgets.remove ( widget )


    def paint (self, force = 0, clear = 0):
        """Repaints the surface if needed."""
        paintneeded = 0

        # should we clear the disply first?
        if clear:
            # yep, do that first
            self.surface.fill ( (0,0,0) ) 
            
        # loop over all widgets
        for widget in self.widgets:
            # paint widget 
            if widget.paint (self.surface, force):
                # it did something, so set the flag
                paintneeded = 1
                
        # is a paint needed?
        if paintneeded:
            # update the whole screen
            pygame.display.flip()
         

    def handle (self, event):
        """Handles the passed event. If the event is of any interest to any of the registred widgets
        the widgets are allowed to act on the event. If any widget handled the event then this
        method will return 1 to indicate that it should not be processed any further. A value of 0
        indicates that the event was not processed."""

        # what type of event do we have?
        if event.type == widget.KEYDOWN:
            # keyboard key pressed 
            return self.handleKeyDown (event)
            
        elif event.type == widget.KEYUP:
            # keyboard key released
            return self.handleKeyUp (event)

        elif event.type == widget.MOUSEMOTION:
            # mouse moved 
            return self.handleMouseMotion (event)

        elif event.type == widget.MOUSEBUTTONDOWN:
            # mouse button pressed
            return self.handleMousePressed (event)

        elif event.type == widget.MOUSEBUTTONUP:
            # mouse button released 
            return self.handleMouseReleased (event)

        elif event.type == widget.USEREVENT:
            # timer event
            return self.handleTimer (event)

        elif event.type == widget.QUIT:
            # player wants to quit
            print "*** player wants to quit, what should we do? ***"
            
        # it was none of our events
        return 0
    

    def handleKeyDown (self, event):
        """Handles an event when a key was pressed but not yet released. Uses the last known
        position of the mouse to determine which widget has focus."""

        # by default we've not handled it
        handled = widget.UNHANDLED

        # loop over all widgets in the list
        for w in self.widgets:
            # is the mouse inside the widget?
            if w.isInside ( self.last_pos ):
                # have the widget handle the event
                w.handle ( widget.KEYDOWN, event )
                handled = widget.HANDLED
 
        return handled
       

    def handleKeyUp (self, event):
        """Handles an event when a pressed key released. Uses the last known
        position of the mouse to determine which widget has focus."""
        # by default we've not handled it
        handled = widget.UNHANDLED

        # loop over all widgets in the list
        for w in self.widgets:
            # is the mouse inside the widget?
            if w.isInside ( self.last_pos ):
                # have the widget handle the event
                w.handle ( widget.KEYUP, event )
                handled = widget.HANDLED

        return handled
     

    def handleMouseMotion (self, event):
        """Handles an event when the mouse pointer was moved within a widget."""

        # get the position of the mouse
        position = event.pos

        # by default we've not handled it
        handled = widget.UNHANDLED
        
        # loop over all widgets in the list
        for w in self.widgets:

            # is the mouse inside the widget?
            if w.isInside ( position ) and not w.isInside ( self.last_pos ):
                # have the widget handle the event
                w.handle ( widget.MOUSEENTEREVENT, event )
                handled = widget.HANDLED

            # is the mouse outside the widget
            elif not w.isInside ( position ) and w.isInside ( self.last_pos ):
                # have the widget handle the event
                w.handle ( widget.MOUSELEAVEEVENT, event )
                handled = widget.HANDLED

            # is the mouse the widget and the last move was too inside?
            elif w.isInside ( position ) and w.isInside ( self.last_pos ):
                # have the widget handle the event
                w.handle ( widget.MOUSEMOTION, event )
                handled = widget.HANDLED

        # store new last position
        self.last_pos = position

        return handled
            

    def handleMousePressed (self, event):
        """Handles an event when a mouse key was pressed but not yet released."""

        # get the position of the press
        position = event.pos
       
        # loop over all widgets in the list
        for w in self.widgets:
            # is the mouse inside the widget?
            if w.isInside ( position ):
                # have the widget handle the event
                status = w.handle ( widget.MOUSEBUTTONDOWN, event )

                # did it handle it?
                if status != widget.UNHANDLED:
                    # it handled it, we're done here
                    return status

        return widget.UNHANDLED
       

    def handleMouseReleased (self, event):
        """Handles an event when a pressed mouse button was released."""

        # get the position of the press
        position = event.pos
        
        # loop over all widgets in the list
        for w in self.widgets:
            # is the mouse inside the widget?
            if w.isInside ( position ):
                # have the widget handle the event
                status = w.handle ( widget.MOUSEBUTTONUP, event )

                # did it handle it?
                if status != widget.UNHANDLED:
                    # it handled it, we're done here
                    return status

        return widget.UNHANDLED


    def handleTimer (self, event):
        """Handles a timer event."""

        # do we have a dialog?
        if self.dialog != None:
            # yep, call it
            return self.dialog.timer ()
            
        # it was handled
        return widget.UNHANDLED
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
